@@ -1,6 +1,6 @@
 use strict;
 package Web::Authenticate::Session::Handler;
-$Web::Authenticate::Session::Handler::VERSION = '0.003';
+$Web::Authenticate::Session::Handler::VERSION = '0.004';
 use Mouse;
 use Carp;
 use DateTime;
@@ -64,7 +64,7 @@ sub delete_session {
     return unless $session_id;
 
     $self->session_storage_handler->delete_session($session_id);
-    $self->cookie_handler->delete_cookie($self->session_id_cookie_name);
+    $self->_delete_session_cookie;
 }
 
 
@@ -92,7 +92,11 @@ sub get_session {
 
     return unless $session_id;
 
-    return $self->session_storage_handler->load_session($session_id);
+    my $session = $self->session_storage_handler->load_session($session_id);
+
+    $self->_delete_session_cookie unless $session;
+
+    return $session;
 }
 
 sub _get_session_id {
@@ -110,6 +114,11 @@ sub _set_session_cookie {
     $self->cookie_handler->set_cookie($self->session_id_cookie_name, $session_id, $self->session_expires_in_seconds);
 }
 
+sub _delete_session_cookie {
+    my ($self) = @_;
+    $self->cookie_handler->delete_cookie($self->session_id_cookie_name);
+}
+
 1;
 
 __END__
@@ -124,7 +133,7 @@ Web::Authenticate::Session::Handler - The default implementation of Web::Authent
 
 =head1 VERSION
 
-version 0.003
+version 0.004
 
 =head1 METHODS
 
@@ -164,7 +173,8 @@ Deletes all sessions for user.
 
 =head2 get_session
 
-Returns the session for the logged in user. Returns undef if there is no session.
+Returns the session for the logged in user. Returns undef if there is no session. If an invalid
+session is found in the user's cookies, that session will be deleted from their cookies.
 
 =head1 AUTHOR
 
